Module: ActiveRecordQueryCounter::ConnectionAdapterExtension::ConnectionSetupExtension
- Defined in:
- lib/active_record_query_counter/connection_adapter_extension.rb
Overview
Module prepended to the connection adapter to measure the wall clock time spent establishing, verifying, or reconnecting the database connection while a query is running (see CONNECTION_SETUP_METHODS). The measured time is accumulated on the current query's connection timer so it can be subtracted from the reported query time.
Instance Method Summary collapse
Instance Method Details
#connect! ⇒ Object
66 67 68 |
# File 'lib/active_record_query_counter/connection_adapter_extension.rb', line 66 def connect!(...) ActiveRecordQueryCounter.measure_connection_setup { super } end |
#reconnect! ⇒ Object
70 71 72 |
# File 'lib/active_record_query_counter/connection_adapter_extension.rb', line 70 def reconnect!(...) ActiveRecordQueryCounter.measure_connection_setup { super } end |
#verify! ⇒ Object
74 75 76 |
# File 'lib/active_record_query_counter/connection_adapter_extension.rb', line 74 def verify!(...) ActiveRecordQueryCounter.measure_connection_setup { super } end |