Module: ActiveRecordQueryCounter::ConnectionAdapterExtension::ConnectionSetupExtension
- Defined in:
- lib/active_record_query_counter/connection_adapter_extension.rb
Overview
Module prepended to the connection adapter to measure the wall clock time spent establishing, verifying, or reconnecting the database connection while a query is running (see CONNECTION_SETUP_METHODS). The measured time is accumulated on the current query's connection timer so it can be subtracted from the reported query time.
Instance Method Summary collapse
Instance Method Details
#connect! ⇒ Object
79 80 81 |
# File 'lib/active_record_query_counter/connection_adapter_extension.rb', line 79 def connect!(...) ActiveRecordQueryCounter.measure_connection_setup { super } end |
#reconnect! ⇒ Object
83 84 85 |
# File 'lib/active_record_query_counter/connection_adapter_extension.rb', line 83 def reconnect!(...) ActiveRecordQueryCounter.measure_connection_setup { super } end |
#verify! ⇒ Object
87 88 89 |
# File 'lib/active_record_query_counter/connection_adapter_extension.rb', line 87 def verify!(...) ActiveRecordQueryCounter.measure_connection_setup { super } end |